{ MineSweeper solver.
  This program solves the MineSweeper game.
  After the user indicated the program which is the MineSweeper Window,
  the program moves alone the mouse cursor to click on cells and
  solve the game.
  In this version, the strategie is not very developped, and there are
  some other tests.
  In  tEx064.pp,  the strategie will be more sophisticated, using
  probabilities to decide on which cell to click.
  --------------------------------------------}
program tEx063;

// To tell FreePascal that the application is a Graphical User Interface
// i.e. is running in windows, not in text mode.
{$APPTYPE GUI}

Uses

  wApplication0
 ,wWindow0
 ,wDialog0
 ,wWindowBase0
 ,wDialogCreatorLow2
 ,wVariableNumLink2
// ,wUtiles0 // for the AccelTableMSGCreate function
 ,wGlobals2
 ,wDebug0
 ,Windows;

type
  PMyWindow = ^TMyWindow;
  TMyWindow = object(TWindow)
  //"""""""""""""""""""""""""
  hWinSec       : integer;
  // Handle of the other window

  fGetWinSecHandle : boolean;
  // To get the window handle of another window

  pDialogActivate : PDialog;

  colGray : TypeColor;
  // Gray color of the cels.

  fSolveFast : boolean;
  // To solve faster and not see the mouse mouvments.

  ptTopLeft : POINT;
  // Position in pixels in the client coordinate of the top left corner
  // of the top left mine square.

  ptRightBottomMargin : POINT;
  // Size of the right and Bottom margin of the Minesweeper game.
  // Used to compute the number of cells in the game.

  nXDelta, nYDelta : integer;
  // number of pixels between two mine squares.

  nXNumber, nYNumber : integer;
  // number of mine squares on a line and on a column

  fFinished : boolean;
  // Set to TRUE if finished

  aabM : array[0..200,0..150] of byte;
  // The content of the MinerSweep
  // Row and column 0 are used to simplify computation, but
  // First mine square Row and Column begins with index 1.

  aabF : array[0..200,0..150] of byte;
  // number of neighbour cels having a Flag, i.e.
  // a desactivated mine.

  aabTS : array[0..200,0..150] of byte;
  // number of neighbour cels Too See, i.e.
  // not having been explored.

  constructor Init(strWinTitlePrm:ShortString);
  destructor Done; virtual;

  procedure ToggleSolveFast;
  // Option, to set the speed

  procedure GetSecWindowHandle;
  // To get the handle of the minesweeper window

  procedure GetSecWindowPosition;
  procedure SimulateLeftClick;
  procedure SimulateRightClick;

  procedure MoveMouseTo(nX2,nY2:integer);
  // Moving the mouse cursor to the position nXP, nYP, slowly

  procedure MoveMouse;
  procedure GetNumbersColors;
  procedure GetSquarePositions;

  function GetMineContent(hDCS:HDC; nXP,nYP:integer):integer;
  // nXP, nYP = screen coordinate of the upper left of a mine square.
  // return the content of it.
  // = cteToSee if it has not been explored, 0 if empty,
  // = cteFlag  if a flag is in
  // = cteBoom  if a bomb is in, and thus the game finished.

  procedure GetMineSweeperContent;

  procedure PutFlags;
  // Put Flags on cells where it is sure that a bomb existe.

  function SaftyExplore:integer;
  // Click on cells which have no bombs for sure

  procedure SolveMineSweeper;
  // Go and click to solve the minesweeper

  procedure AskAcceptDlgQuit(poDlg:PWindowBase0; wCtrlIDP:word32; var wRep:word32); virtual;
  // To set to NIL the pointer on the dialog box

  function WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  // Handles the menu messages

  function WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  end; // TMyWindow = object(TWindow)

const
  cteToSee = 20;  // unexplored cells
  cteFlag  = 21;  // A cells with a bomb has been marked.
  cteBoom  = 99;  // A bomb has been found

var
  oApp        : TApplication;
  poMyMainWin : PMyWindow;

// var nNotUsed : integer; // only to avoid an error message while compliling in DevPascal 1.9

constructor TMyWindow.Init(strWinTitlePrm:ShortString);
//=====================================================
begin
inherited Init(strWinTitlePrm);

Int32Field(  1001, ptTopLeft.x);
Int32Field(  1002, ptTopLeft.y);
Int32Field(  1003, ptRightBottomMargin.x);
Int32Field(  1004, ptRightBottomMargin.y);
IntegerField(1005, nXDelta);
IntegerField(1006, nYDelta);
IntegerField(1007, nXNumber);
IntegerField(1008, nYNumber);
BooleanField(1009, fSolveFast);

hWinSec:=0;
fGetWinSecHandle:=FALSE;
pDialogActivate:=NIL;
fSolveFast :=FALSE;

// Usually these values are fixed.
// They can be changed by the user, or computed by the program
ptTopLeft.x  :=14;
ptTopLeft.y  :=56;
ptRightBottomMargin.x:= 8; // 2 pixels less, as a security
ptRightBottomMargin.y:= 8;
nXDelta  :=16;
nYDelta  :=16;
nXNumber :=8;  // This can change, depending on the size of the window
nYNumber :=8;  // This can change, depending on the size of the window

// This line is all what is needed to have a status line.
new(poStatusLine, Init(@Self));
end;  // TMyWindow.Init

destructor TMyWindow.Done;
//=========================
begin
inherited Done;
end;  // TMyWindow.Done

procedure TMyWindow.ToggleSolveFast;
//==================================
// Option, to set the speed
var
  wCheck      // to set a check mark in a menu.
              : word;
begin
// Get the handle on the menu
// hMenuWin:=GetMenu(hWindow);  already done

// Toggle from fast solving to slow solwing, showing the mouse movements
fSolveFast:=not fSolveFast;

// change the menu check mark
if fSolveFast then wCheck:=MF_CHECKED
else               wCheck:=MF_UNCHECKED;
CheckMenuItem(hMenuWin, 351, MF_BYCOMMAND or wCheck);
end;  // TMyWindow.ToggleSolveFast

procedure TMyWindow.GetSecWindowHandle;
//=====================================
// Get the handle of the window which is being activated
begin
// By clicking on another window, normally the MineSweeper window,
// Gets the handle of that window.
hWinSec := 0;

// These two values indicate that when the main window is desactivated,
// The new active window is the wanted one.
pDialogActivate:=PDialog(DialogCreateMSG(20199, @Self));
fGetWinSecHandle:=TRUE;

SetActiveWindow(hWin);
SetFocus(hWin);
end;  // TMyWindow.GetSecWindowHandle

procedure TMyWindow.GetSecWindowPosition;
//=======================================
// To get the position and size of the Second window, i.e. the window
// which was clicked before
var rRec : RECT;
begin
if (hWinSec = 0) then begin
   DialogExecMSG(20299, @Self);
   exit;
   end;

// Get the position and size of the Minesweeper window
GetWindowRect(hWinSec, rRec);
DBW(2, 'Screen Position =' + ITS5(rRec.Left) + ITS5(rRec.Top));
DBW(2, 'Window Size X,Y =' + ITS5(rRec.Right - rRec.Left)
                           + ITS5(rRec.Bottom - rRec.Top));
GetClientRect(hWinSec, rRec);
DBW(2, 'Client Size X,Y =' + ITS5(rRec.Right) + ITS5(rRec.Bottom));

// If incoherent values, put standard values
if (nXDelta <=0) then nXDelta:=16;
if (nYDelta <=0) then nYDelta:=16;

// Number of cells
nXNumber:=(rRec.Right  - ptTopLeft.x - ptRightBottomMargin.x) div nXDelta;
nYNumber:=(rRec.Bottom - ptTopLeft.y - ptRightBottomMargin.y) div nYDelta;

DBW(4, 'Number of Cells X,Y =' + ITS5(nXNumber) + ITS5(nYNumber));
end;  // TMyWindow.GetSecWindowPosition

procedure TMyWindow.SimulateLeftClick;
//====================================
// Just to simulate a Left Click on the Minesweeper window.
var
  ptL         : TPoint;
begin
if (hWinSec = 0) then begin
   DialogExecMSG(20299, @Self);
   exit;
   end;

// Get the mouse position, to transmit it as a parameter to the message
GetCursorPos(@ptL);
ScreenToClient(hWinSec, ptL);

SetForegroundWindow(hWinSec);
SendMessage(hWinSec, WM_LBUTTONDOWN, MK_LBUTTON, MakeLong(ptL.x, ptL.y));
SendMessage(hWinSec, WM_LBUTTONUP,            0, MakeLong(ptL.x, ptL.y));

// Set back the main window as the active one.
SetForegroundWindow(hWin);
end;  // TMyWindow.SimulateLeftClick

procedure TMyWindow.SimulateRightClick;
//=====================================
// Just to simulate a Left Click on the Minesweeper window.
var ptL         : TPOINT;
begin
if (hWinSec = 0) then begin
   DialogExecMSG(20299, @Self);
   exit;
   end;

// Get the mouse position, to transmit it as a parameter to the message
GetCursorPos(@ptL);
ScreenToClient(hWinSec, ptL);

SetForegroundWindow(hWinSec);
SendMessage(hWinSec, WM_RBUTTONDOWN, MK_RBUTTON, MakeLong(ptL.x, ptL.y));
SendMessage(hWinSec, WM_RBUTTONUP,            0, MakeLong(ptL.x, ptL.y));

// Set back the main window as the active one.
SetForegroundWindow(hWin);
end;  // TMyWindow.SimulateRightClick

procedure TMyWindow.MoveMouseTo(nX2,nY2:integer);
//===============================================
// Moving the mouse cursor to the position nXP, nYP,
// slowly of fast, depending on  fSolveFast
var
  nSecurity,
  nXStep,nYStep,
  nX1,nY1,
  nX,nY          : int32;
  wTime0      : word32;
  ptL         : TPOINT;
begin
if fSolveFast then begin
   // Solving fast, do not show the mouse cursor movements.
   SetCursorPos(nX2,nY2);
   exit;
   end;

GetCursorPos(@ptL);
nX1:=ptL.x;  nY1:=ptL.y;  // Starting position
nX:=nX1;     nY:=nY1;     // Starting position

if (nX1 = nX2) and (nY1 = nY2) then exit; // nothing to do

// To set the moving direction
if (nX2 > nX1) then nXStep:=1 else if (nX2 < nX1) then nXStep:=-1 else nXStep:=0;
if (nY2 > nY1) then nYStep:=1 else if (nY2 < nY1) then nYStep:=-1 else nYStep:=0;

nSecurity:=2000;
repeat
   // Move the mouse allong a strait line.
   // Must take care if (X1=X2) or if (Y1=Y2)
   if (abs((nY2-nY)*(nX2-nX1)) < abs((nX2-nX)*(nY2-nY1))) then nX:=nX + nXStep
   else if (abs((nY2-nY)*(nX2-nX1)) > abs((nX2-nX)*(nY2-nY1))) then nY:=nY + nYStep
   else if (nX <> nX2) then nX:=nX + nXStep
   else if (nY <> nY2) then nY:=nY + nYStep;
   // else do not move, the mouse cursor is already at destination.

   // Moves the cursor
   SetCursorPos(nX,nY);

   // Wait to let us see the movement
   wTime0:=GetTickCount;
   repeat until wTime0 <> GetTickCount;

   dec(nSecurity);
   until ((nX = nX2) and (nY = nY2)) or (nSecurity <= 0);

if (nSecurity <= 0) then
   DBW(1, '***** PROBLEM, X1,Y1,X2,Y2=' + ITS5(nX1) + ITS5(nY1) + ITS5(nX2) + ITS5(nY2));
end;  // TMyWindow.MoveMouseTo

procedure TMyWindow.MoveMouse;
//============================
// Moving the mouse cursor, just a test.
var
  nX2,nY2     : int32;
  rRec        : RECT;
begin
if (hWinSec <> 0) then begin
   GetWindowRect(hWinSec, rRec);
   nX2:=rRec.Right-20;   nY2:=rRec.Bottom-20; // destination
   end
else begin  nX2:=798;   nY2:=598; end; // default destination

MoveMouseTo(nX2,nY2);
end;  // TMyWindow.MoveMouse

procedure TMyWindow.GetNumbersColors;
//===================================
// To get the color of the numbers, just a test
var
  ptL         : TPoint;
  hDCScreen   : HDC;
  colPixel    : TypeColor;
begin
DialogExecMSG(20399, @Self);

// Get Mouse position
GetCursorPos(@ptL);

// Get a device context of the screen
hDCScreen:=GetDC(0);

colPixel:= GetPixel(hDCScreen, ptL.x, ptL.y);

ReleaseDC(0, hDCScreen);

DBW(1, 'color=' + ITS5(colPixel.bRed) + ITS5(colPixel.bGreen) + ITS5(colPixel.bBlue));
end;  // TMyWindow.GetNumbersColors

procedure TMyWindow.GetSquarePositions;
//=====================================
// Get the positions, sizes of all mine squares, and how many they are.
var
  nSecurity,
  nX,nY       : int32;
  wTime0      : word32;
  rRec        : RECT;
  hDCScreen   : HDC;
  colLast,    // Last pixel color
  colPixel    : TypeColor;
  nCountSame // count the number of colors which where the same on a line
              : integer;
  nYBottomL   // bottom of the last mine square
              : integer;
begin
if (hWinSec = 0) then begin
   // A second window must have been selected
   DialogExecMSG(20299, @Self);
   exit;
   end;

// Initialisations
ptTopLeft.x:=0;
ptTopLeft.y :=0;   nYBottomL:=0;
nXDelta :=0;
nYDelta :=0;
nXNumber :=0;
nYNumber :=0;

// To get a starting point
GetWindowRect(hWinSec, rRec);
nX:=rRec.Left;
nY:=rRec.Bottom-20;

// Get a device context of the screen
hDCScreen:=GetDC(0);

// Get first pixel color
colLast:= GetPixel(hDCScreen, nX, nY);

nCountSame:=1;

nSecurity:=2000;
repeat
   nX:=nX + 1;

   // Moves the cursor
   SetCursorPos(nX,nY);

   colPixel:= GetPixel(hDCScreen, nX, nY);

   if (colPixel = colLast) then inc(nCountSame)
   else begin
      DBW(16, 'nb, color=' + ITS5(nCountSame) + ITS5(colLast.bRed)
                          + ITS5(colLast.bGreen) + ITS5(colLast.bBlue));

      if (nCountSame >= 10) then begin
         // a mine square has been passed through
         if (ptTopLeft.x = 0) then ptTopLeft.x:=nX-nCountSame
         else if (nXDelta = 0) then nXDelta:=nX-nCountSame - ptTopLeft.x;

         inc(nXNumber);
         end;

      nCountSame:=1;
      colLast:=colPixel;
      end;

   // Wait to let us see the movement
   wTime0:=GetTickCount;
   repeat until wTime0 <> GetTickCount;

   until (nX >= rRec.Right) or (nSecurity <= 0);

// Loop to count the mine squares vertically
//"""""""""""""""""""""""""""""""""""""""""""
nX:=rRec.Left+20;
nY:=rRec.Bottom;

// Get first pixel color
colLast:= GetPixel(hDCScreen, nX, nY);

nCountSame:=1;

nSecurity:=2000;
repeat
   nY:=nY - 1;

   // Moves the cursor
   SetCursorPos(nX,nY);

   colPixel:= GetPixel(hDCScreen, nX, nY);

   if (colPixel = colLast) then inc(nCountSame)
   else begin
      DBW(16, 'nb, color=' + ITS5(nCountSame) + ITS5(colLast.bRed)
                          + ITS5(colLast.bGreen) + ITS5(colLast.bBlue));

      if (nCountSame >= 10) then begin
         // a mine square has been passed through
         if (nYBottomL = 0) then nYBottomL:=nY
         else if (nYDelta = 0) then nYDelta:=nYBottomL -(nY);

         inc(nYNumber);
         end;

      nCountSame:=1;
      colLast:=colPixel;
      end;

   // Wait to let us see the movement
   wTime0:=GetTickCount;
   repeat until wTime0 <> GetTickCount;

   until (nY <= rRec.Top+80) or (nSecurity <= 0);

ReleaseDC(0, hDCScreen);

ptTopLeft.y:=nYBottomL - nYDelta * (nYNumber-1);

// Moves the cursor to see the left top corner of the left top mine square
SetCursorPos(ptTopLeft.x,ptTopLeft.y);

// Convert to have coordinate in client units.
ScreenToClient(hWinSec, ptTopLeft);

// Determine the margins
GetClientRect(hWinSec, rRec);
ptRightBottomMargin.x:= rRec.Right  - ptTopLeft.x - nXDelta*nXNumber;
ptRightBottomMargin.y:= rRec.Bottom - ptTopLeft.y - nXDelta*nYNumber;

DBW(8, 'nXLeft, nXDelta, nXNumber=' + ITS5(ptTopLeft.x) + ITS5(nXDelta) + ITS5(nXNumber));
DBW(8, 'nYTop, nYDelta, nYNumber =' + ITS5(ptTopLeft.y) + ITS5(nYDelta) + ITS5(nYNumber));
end;  // TMyWindow.GetSquarePositions

function TMyWindow.GetMineContent(hDCS:HDC; nXP,nYP:integer):integer;
//===================================================================
// nXP, nYP = screen coordinate of the upper left of a mine square.
// return the content of it.
// = cteToSee if it has not been explored, 0 if empty,
// = cteFlag  if a flag is in
// = cteBoom  if a bomb is in, and thus the game finished.
var
  colPixel1
 ,colPixel2
            : TypeColor;

begin
// Get some pixels, to see the content of the mine square
colPixel1:= GetPixel(hDCS, nXP, nYP);

//DBW(1, ITS5(nXP) + ITS5(nYP));
//SetPixel(hDCS, -1 + nXP, nYP, RGB(0,0,0));

if (colPixel1 <> colGray) then begin
   // It is either an unexplored square, of a desactivated square
   colPixel2:= GetPixel(hDCS,  4 + nXP, 3 + nYP);

   if (colPixel2 <> colGray) then begin
      // A desactivated cell
      GetMineContent:=cteFlag;
      exit;
      end;

   // An unexplored cell
   GetMineContent:=cteToSee;
   exit;
   end;

// Here we must determine the number of bombs around this square,
// i.e. read the number in the square, or return zero.

colPixel1:= GetPixel(hDCS,  7 + nXP,  7 + nYP);

if (colPixel1 = colGray) then begin
   // An empty square
   GetMineContent:=50;
   exit;
   end;

colPixel1:= GetPixel(hDCS,  3 + nXP,  3 + nYP);

if (colPixel1 = colGray) then begin
   // Not a  2,  not a  3, not a 4, not a 5
   colPixel2:= GetPixel(hDCS,  4 + nXP,  7 + nYP);

   if (colPixel2 <> colGray) then begin
      // Not a 1
      GetMineContent:=cteBoom;
      exit;
      end;

   GetMineContent:=1;
   exit;
   end;

colPixel2:= GetPixel(hDCS,  4 + nXP,  9 + nYP);
if (colPixel2 = colGray) then begin
   // not a  2
   colPixel2:= GetPixel(hDCS,  3 + nXP,  10 + nYP);
   if (colPixel2 = colGray) then begin
      // not a  3
      GetMineContent:=4;
      exit;
      end;

   colPixel2:= GetPixel(hDCS,  3 + nXP,  6 + nYP);
   if (colPixel2 = colGray) then begin
      // not a  3
      GetMineContent:=3;
      exit;
      end;

   GetMineContent:=5;
   exit;
   end;

GetMineContent:=2;
end;  // TMyWindow.GetMineContent

procedure TMyWindow.GetMineSweeperContent;
//========================================
// Get the content of the complete minesweeper window
var
  ptTLScreenL : POINT; // top left in screen coordinate of the upper left mine square
  nContent,   // Content of a mine square
  nX,nY,
  ii,jj       : integer;
  hDCScreen   : HDC;
  strS        : ShortString;
begin
if (hWinSec = 0) then begin
   DialogExecMSG(20299, @Self);
   exit;
   end;

if (nXDelta = 0) then GetSquarePositions;
DBW(32, '--------------------------------------------');

// initialisation of the aabM content.
for nX:=0 to nXNumber + 1 do begin
   aabM[nX,0]          :=0; // Unexplored
   aabM[nX,nYNumber+1] :=0;
   end;

for nY:=0 to nYNumber + 1 do begin
   aabM[0,nY]          :=0; // Unexplored
   aabM[nXNumber+1,nY] :=0;
   end;

for nX:=0 to nXNumber + 1 do
  for nY:=0 to nYNumber + 1 do begin
   aabF[nX,nY]          := 0; // Unexplored
   aabTS[nX,nY]         := 0; // Unexplored
   end;

// Get a device context of the screen
hDCScreen:=GetDC(0);

ptTLScreenL:=ptTopLeft;
ClientToScreen(hWinSec, ptTLScreenL);

// Reference Gray color
colGray:=GetPixel(hDCScreen, 1+ptTLScreenL.x, 1+ptTLScreenL.y);

// If no unexplored cells are found, its finished
fFinished:=TRUE;

// Loop on all mine squares to see their content
nY:=1;
repeat
   nX:=1;

   // Loop on a row
   repeat
      // Get the content of the cell (nX, nY)
      nContent:=GetMineContent(hDCScreen, ptTLScreenL.x + (nX-1)*nXDelta,
                                          ptTLScreenL.y + (nY-1)*nYDelta);

      // To count the neighbour cell to be still explored
      if (nContent = cteToSee) then begin
         fFinished:=FALSE;
         for ii:=nX-1 to nX+1 do  for jj:=nY-1 to nY+1 do
           aabTS[ii,jj]:=aabTS[ii,jj] + 1;

         aabTS[nX,nY]:=aabTS[nX,nY] - 1;
         end;

      // To count the neighbour cell having a marked bomb
      if (nContent = cteFlag) then begin
         for ii:=nX-1 to nX+1 do  for jj:=nY-1 to nY+1 do
           aabF[ii,jj]:=aabF[ii,jj] + 1;

         aabF[nX,nY]:=aabF[nX,nY] - 1;
         end;

      // Content of this cell
      aabM[nX,nY]:=nContent;

      inc(nX);
      until (nX > nXNumber);

   inc(nY);
   until (nY > nYNumber);

ReleaseDC(0, hDCScreen);

// To show what has been found in the squares
for nY:=1 to nYNumber do begin
   strS:='|';
   for nX:=1 to nXNumber do begin
      if (aabM[nX,nY] = cteFlag) then // A flgs is in
        strS:=strS + ';'
      else if (aabM[nX,nY] = cteToSee) then // An untouched square
        strS:=strS + '.'
      else if (aabM[nX,nY] = cteBoom) then // A bomb
        begin strS:=strS + '*';  aabM[0,0]:=cteBoom; end // lost
      else if (aabM[nX,nY] = 50) then
        strS:=strS + ' '
      else
        strS:=strS + IntToStr(aabM[nX,nY],1);

      strS:=strS + IntToStr(aabTS[nX,nY],1);
      strS:=strS + IntToStr(aabF[nX,nY],1) + ' ';

      end; // for nX

   DBW(64, strS + '|');
   end; // for nY
end;  // TMyWindow.GetMineSweeperContent

procedure TMyWindow.PutFlags;
//===========================
// Put Flags on cells where it is sure that a bomb existe.
var
  ptTLScreenL : POINT; // top left in screen coordinate of the upper left mine square
  nX,nY,
  ii,jj       : integer;
begin
if (hWinSec = 0) then begin
   DialogExecMSG(20299, @Self);
   exit;
   end;

ptTLScreenL:=ptTopLeft;
ClientToScreen(hWinSec, ptTLScreenL);

// To show what has been found in the squares
for nY:=1 to nYNumber do begin
   for nX:=1 to nXNumber do begin
      if (aabM[nX,nY] = aabF[nX,nY] + aabTS[nX,nY]) then begin
DBW(128, 'SetFlag' + ITS5(nX) + ITS5(nY));
         // neighbour of this cell have a bomb
         for ii:=nX-1 to nX+1 do  for jj:=nY-1 to nY+1 do begin
            if (aabM[ii,jj] = cteToSee) then begin
               MoveMouseTo((ii-1)*nXDelta + ptTLScreenL.x+4,
                           (jj-1)*nYDelta + ptTLScreenL.y+4);
               SimulateRightClick;
               aabM[ii,jj]:=cteFlag;
               end;
            end;
         end;
      end; // for nX
   end; // for nY

GetMineSweeperContent;
end;  // TMyWindow.PutFlags

function TMyWindow.SaftyExplore:integer;
//======================================
// Click on cells which have no bombs for sure
// Return the number of cells which have been clicked.
var
  ptTLScreenL : POINT; // top left in screen coordinate of the upper left mine square
  nCount,     // Count how many cells have been clicked
  nX,nY,
  ii,jj       : integer;
begin
if (hWinSec = 0) then begin
   DialogExecMSG(20299, @Self);
   exit;
   end;

ptTLScreenL:=ptTopLeft;
ClientToScreen(hWinSec, ptTLScreenL);

nCount:=0;

// To show what has been found in the squares
for nY:=1 to nYNumber do begin
   for nX:=1 to nXNumber do begin
      if (aabM[nX,nY] = aabF[nX,nY]) then begin
         // neighbour of this cell have no bombs for sure
DBW(128, 'Explore' + ITS5(nX) + ITS5(nY));
         for ii:=nX-1 to nX+1 do  for jj:=nY-1 to nY+1 do begin
            if (aabM[ii,jj] = cteToSee) then begin
               MoveMouseTo((ii-1)*nXDelta + ptTLScreenL.x+4,
                           (jj-1)*nYDelta + ptTLScreenL.y+4);
               SimulateLeftClick;
               aabM[ii,jj] := cteToSee+1; // do not click again
               inc(nCount);
               end;
            end;
         end;
      end; // for nX
   end; // for nY

GetMineSweeperContent;
SaftyExplore:=nCount;
end;  // TMyWindow.SaftyExplore

procedure TMyWindow.SolveMineSweeper;
//===================================
// Go and click to solve the minesweeper
// This will be better done in  tEx064.pp
var
  ptTLScreenL : POINT; // top left in screen coordinate of the upper left mine square
  nSecurity2,
  nSecurity   : integer;
  nXL,nYL     : integer;
  fDidOneClickL : boolean;
begin
if (hWinSec = 0) then begin
   DialogExecMSG(20299, @Self);
   exit;
   end;

// Screen position of the cells
ptTLScreenL:=ptTopLeft;
ClientToScreen(hWinSec, ptTLScreenL);
SetCursorPos(ptTLScreenL.x, ptTLScreenL.y);

// A first click can be usefull
SetCursorPos(ptTLScreenL.x-4, ptTLScreenL.y-4);
SimulateLeftClick;

// Get the cells content.
GetMineSweeperContent;

// Loop to solve
nSecurity:=100;
repeat
   // Try to put flags on cells having bombs
   PutFlags;

   // Try to desactivate bombs. If none is found, choose one cell at random
   if (SaftyExplore = 0) then begin
     fDidOneClickL:=FALSE;
     nSecurity2:=500; // To not loop forever
     repeat
      // must try at random
      nXL:=Random(nXNumber);
      nYL:=Random(nYNumber);

      // If an unexplored cell has been found, click on it
      if (aabM[nXL+1, nYL+1] = cteToSee) then begin
         MoveMouseTo(nXL*nXDelta + ptTLScreenL.x+4, nYL*nYDelta + ptTLScreenL.y+4);
         SimulateLeftClick;
         GetMineSweeperContent;
         fDidOneClickL:=TRUE;
         end;

       dec(nSecurity2);
       until fDidOneClickL or (nSecurity2 <= 0);
      end;

   dec(nSecurity);
   until (aabM[0,0] = cteBoom) or (nSecurity <= 0) or fFinished or
         (GetAsyncKeyState(VK_ESCAPE) < 0);
end;  // TMyWindow.SolveMineSweeper

function TMyWindow.WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT;
//=============================================================
// Handles the menu messages
Var wNrMenu     : word;
begin
WMCommand := 0; // Default return value when processing a message

if (lPar <> 0) then begin
   // call the parent WMCommand method.
   WMCommand:=inherited WMCommand(wPar, lPar);
   exit;
   end;

// for the menus,  lPar = 0
//=========================
wNrMenu   := LoWord(wPar);
Case wNrMenu Of
   101 : GetSecWindowHandle;
   102 : GetSecWindowPosition;
   103 : SimulateLeftClick;
   104 : SimulateRightClick;
   105 : MoveMouse;
   106 : GetNumbersColors;
   107 : GetSquarePositions;
   108 : GetMineSweeperContent;
   109 : SolveMineSweeper;

   111 : PutFlags;
   112 : SaftyExplore;

   // Parameters
   301 : DialogExecMSG(30199, @Self);
   351 : ToggleSolveFast;

   // A help information
   901 : DialogExecMSG(90199, @Self);

   else WMCommand:=inherited WMCommand(wPar, lPar);
   end;  // Case NrMenu Of
end;  // TMyWindow.WMCommand

procedure TMyWindow.AskAcceptDlgQuit(poDlg:PWindowBase0; wCtrlIDP:Word32; var wRep:word32);
//=========================================================================================
// notification that the dialog box pointed by  PDialog(lPar)
// is about to quit, accepting its changes (With OK).
// If we refuse to quit, just Set  wRep:=0;
begin
wCtrlIDP:=wCtrlIDP;  wRep:=wRep;  // to avoid a hint
// notification that the dialog box pointed by  pDlg is about to quit.
// Its the best place to check if we accept to quit.
if (poDlg = PWindowBase0(pDialogActivate)) then begin
   // Closing the dialog Box, sets the pointer on it to NIL.
   pDialogActivate:=NIL;
   fGetWinSecHandle:=FALSE;
   end; // if (poDlg = pDialogActivate) then begin
end;  // TMyWindow.AskAcceptDlgQuit

function TMyWindow.WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT;
//======================================================================================
begin
if (wMsg = WM_ACTIVATE) and (LoWord(wPar) = WA_INACTIVE) and
   fGetWinSecHandle and (pDialogActivate <> NIL) and
   (lPar <> pDialogActivate^.hWin) then begin
   // Get the handle of the second window, usually the MineSweeper
   hWinSec:=GetForegroundWindow;
   fGetWinSecHandle:=FALSE;

   // Closes the dialog Box
   DestroyWindow(pDialogActivate^.hWin);
   pDialogActivate:=NIL;

   // Get the position and size of the Minesweeper, and
   // deduce from it the number of cells
   GetSecWindowPosition;
   end;

// Default treatment of a message. Does nearly nothing
WindowProc := inherited WindowProc(hWindow, wMsg, wPar, lPar);
end;  // TMyWindow.WindowProc

Begin // MAIN ******
// Name under which the main window is registered. Spaces are not allowed
oApp.Init('Example063');

// Creation of the main window
new(poMyMainWin, Init('tEx063. Solving the MineSweeper game.'));

// The App.WindowClass can be modified here, if wanted. (Not usual)
oApp.Run;
oApp.Done;  // The main window object is disposed here.
End.

