{ Testing ideas for characterising non locality.

  --------------------------------------------}
program tzNonLocal01;

// To tell FreePascal that the application is a Graphical User Interface
// i.e. is running in windows, not in text mode.
{$APPTYPE GUI}

Uses

  wApplication0
 ,wWindow0
 ,xVectR3
 ,wGlobals2
 ,wDebug0
 ,Windows;

type
  TArray4 = array[1..4,1..4,1..4,1..4] of Float;
  TArray2 = array[1..4,1..4] of Float;
  PMyWindow = ^TMyWindow;
  TMyWindow = object(TWindow)
  //"""""""""""""""""""""""""
  avPr : TArray4; // Probabilities from the experimental results
  avP1, avP2 : TArray2; // Marginal probabilities
  avFF : TArray4; // Functions of the random variables of Alice and Bob

  constructor Init(strWinTitlePrm:ShortString);
  destructor Done; virtual;

  function FuncF(iat,ibt,jat,jbt:integer; var avX:TArray4):Float;
  // Min of Sum(P(ia,ib,ja,jb) - delta(A(ia),ja)*delta(B(ib),jb)) * avV[ia,ib,ja,jb]

  procedure Test1;
  procedure Test2;
  procedure Test3;
  procedure Test4;

  function WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  // Handles the menu messages

  function WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT; virtual;
  end; // TMyWindow = object(TWindow)

var
  oApp        : TApplication;
  poMyMainWin : PMyWindow;
  glf         : boolean; // necessary to use a special assignment operator

constructor TMyWindow.Init(strWinTitlePrm:ShortString);
//=====================================================
begin
inherited Init(strWinTitlePrm);

// This line is all what is needed to have a status line.
new(poStatusLine, Init(@Self));
end;  // TMyWindow.Init

destructor TMyWindow.Done;
//=========================
begin
inherited Done;
end;  // TMyWindow.Done

procedure TMyWindow.Test1;
//========================
var
  iat,ibt,jat,jbt : integer;
  ia,ib,ja,jb     : integer;
  ja1,ja2,jb1,jb2 : integer;
  avThetaA,
  avThetaB    : array[1..5] of Float;
  avecSetA,
  avecSetB    : array[1..5] of RVectR3Float;
  vSign,
  vNoise,
  vLambda,
  vMean,
  vRealis     // One realisation
              : Float;
begin
iat:=2; // Number of settings
ibt:=2;
jat:=2;
jbt:=2;

// Defining the settings
avThetaA[1] := 0.0;
aVThetaA[2] := pi/2;
avThetaB[1] := pi/4;
aVThetaB[2] :=-pi/4;

for ia:=1 to iat do
   avecSetA[ia]:=V3Assign(0.0, cos(avThetaA[ia]), sin(avThetaA[ia]));

for ib:=1 to ibt do
   avecSetB[ib]:=V3Assign(0.0, cos(avThetaB[ib]), sin(avThetaB[ib]));

// Noise in the entangled state
vNoise:=1-1/sqrt(2); // = Limit
vNoise:=0.0;
vLambda:=1.0 - vNoise;

// Defining the Probabilities
DBW(1, '--- Pr --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         if odd(ja+jb) then vSign:= 1
         else               vSign:=-1;
         avPr[ia,ib,ja,jb]:=0.25*(vLambda*(1.0 - vSign * avecSetA[ia]*avecSetB[ib]) + vNoise);

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avPr[ia,ib,ja,jb]));
         end;

// Marginal probabilities on Alice side
DBW(1, '--- P1 --------');
for ia:=1 to iat do
    for ja:=1 to jat do begin
      avP1[ia,ja]:=0;
      for jb:=1 to jbt do avP1[ia,ja]:=avP1[ia,ja] + avPr[ia,1,ja,jb];

      DBW(1, ITS5(ia) + ITS5(ja) + FTS(avP1[ia,ja]));
      end;

// Marginal probabilities on Bob side
DBW(1, '--- P2 --------');
for ib:=1 to ibt do
    for jb:=1 to jbt do begin
      avP2[ib,jb]:=0;
      for ja:=1 to jat do avP2[ib,jb]:=avP2[ib,jb] + avPr[1,ib,ja,jb];

      DBW(1, ITS5(ib) + ITS5(jb) + FTS(avP2[ib,jb]));
      end;

// Try to define the function of the random variables
DBW(1, '--- FF --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         avFF[ia,ib,ja,jb]:=(avPr[ia,ib,ja,jb] - avP1[ia,ja] * avP2[ib,jb]);

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avFF[ia,ib,ja,jb]));
         end;

// Esperance of the combination of the random variables
vMean:=0;
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         vMean:=vMean + avFF[ia,ib,ja,jb] * avPr[ia,ib,ja,jb];
         end;

// Comparaison between the Esperance and the realisations.
DBW(1, '-- Mean  Realis ---------');
for ja1:=1 to jat do
  for ja2:=1 to jat do
    for jb1:=1 to jbt do
      for jb2:=1 to jbt do begin
        // Compute one realisation
        vRealis:= avFF[1,1,ja1,jb1] + avFF[1,2,ja1,jb2] +
                  avFF[2,1,ja2,jb1] + avFF[2,2,ja2,jb2];

        DBW(1, ITS5(ja1) + ITS5(ja2) + ITS5(jb1) + ITS5(jb2) +
               FTS(vMean) + FTS(vRealis));
        end;


end;  // TMyWindow.Test1

procedure TMyWindow.Test2;
//========================
// 3 Settings, 2 possible results.
var
  iat,ibt,jat,jbt : integer;
  ia,ib,ja,jb     : integer;
  ia1,ib1         : integer;
  fStopLoop       : boolean;
  anA,anB         : array[1..5] of integer; // Possible experiment results
  avThetaA,
  avThetaB    : array[1..5] of Float;
  avecSetA,
  avecSetB    : array[1..5] of RVectR3Float;
  vSign,
  vNoise,
  vLambda,
  vMean,
  vRealis     // One realisation
              : Float;
begin
iat:=3; // Number of settings
ibt:=3;
jat:=2;
jbt:=2;

// Defining the settings
avThetaA[1] := 0.0;
aVThetaA[2] :=   pi/3;
aVThetaA[3] := 2*pi/3;
avThetaB[1] := pi-2*pi/6;
aVThetaB[2] := pi;
avThetaB[3] := pi+2*pi/6;

for ia:=1 to iat do
   avecSetA[ia]:=V3Assign(0.0, cos(avThetaA[ia]), sin(avThetaA[ia]));

for ib:=1 to ibt do
   avecSetB[ib]:=V3Assign(0.0, cos(avThetaB[ib]), sin(avThetaB[ib]));

// Noise in the entangled state
vNoise:=0.0;
vLambda:=1.0 - vNoise;

// Defining the Probabilities
DBW(1, '--- Pr --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         if odd(ja+jb) then vSign:= 1
         else               vSign:=-1;
         avPr[ia,ib,ja,jb]:=0.25*(vLambda*(1.0 - vSign * avecSetA[ia]*avecSetB[ib]) + vNoise);

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avPr[ia,ib,ja,jb]));
         end;

// Marginal probabilities on Alice side
DBW(1, '--- P1 --------');
for ia:=1 to iat do
    for ja:=1 to jat do begin
      avP1[ia,ja]:=0;
      for jb:=1 to jbt do avP1[ia,ja]:=avP1[ia,ja] + avPr[ia,1,ja,jb];

      DBW(1, ITS5(ia) + ITS5(ja) + FTS(avP1[ia,ja]));
      end;

// Marginal probabilities on Bob side
DBW(1, '--- P2 --------');
for ib:=1 to ibt do
    for jb:=1 to jbt do begin
      avP2[ib,jb]:=0;
      for ja:=1 to jat do avP2[ib,jb]:=avP2[ib,jb] + avPr[1,ib,ja,jb];

      DBW(1, ITS5(ib) + ITS5(jb) + FTS(avP2[ib,jb]));
      end;

// Try to define the function of the random variables
DBW(1, '--- FF --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         avFF[ia,ib,ja,jb]:=(avPr[ia,ib,ja,jb] - avP1[ia,ja] * avP2[ib,jb]);

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avFF[ia,ib,ja,jb]));
         end;

// Esperance of the combination of the random variables
vMean:=0;
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         vMean:=vMean + avFF[ia,ib,ja,jb] * avPr[ia,ib,ja,jb];
         end;

// Comparaison between the Esperance and the realisations.
DBW(1, '-- Mean  Realis ---------');
for ia:=1 to iat do anA[ia]:=1;
for ib:=1 to ibt do anB[ib]:=1;

ib1:=1;
repeat
   // Compute one realisation
   vRealis:=0;
   for ia:=1 to iat do
      for ib:=1 to ibt do
         vRealis:=vRealis + avFF[ia,ib,anA[ia],anB[ib]];

   DBW(1, FTS(vMean) + FTS(vRealis));

   // Next possible realisation
   ia1:=1;
   repeat
      fStopLoop:=TRUE;
      inc(anA[ia1]);
      if (anA[ia1] > jat) then begin
         anA[ia1]:=1;
         inc(ia1);
         fStopLoop:=FALSE;
         end;
      until fStopLoop or (ia1 > iat);

   if (ia1 > iat) then begin
      ib1:=1;
      repeat
         fStopLoop:=TRUE;
         inc(anB[ib1]);
         if (anB[ib1] > jbt) then begin
            anB[ib1]:=1;
            inc(ib1);
            fStopLoop:=FALSE;
            end;
         until fStopLoop or (ib1 > ibt);
      end;

   until (ib1 > ibt);


end;  // TMyWindow.Test2

function TMyWindow.FuncF(iat,ibt,jat,jbt:integer; var avX:TArray4):Float;
//=======================================================================
// Min of Sum(P(ia,ib,ja,jb) - delta(A(ia),ja)*delta(B(ib),jb)) * avV[ia,ib,ja,jb]
var
  ia,ib,ja,jb     : integer;
  ia1,ib1         : integer;
  fStopLoop       : boolean;
  anA,anB         : array[1..5] of integer; // Possible experiment results
  vSum,
  vMin            : Float;
begin
vSum:=0;
// To do a huge sum, over iat*ibt indexes, each from 1 to jat  or  1 to jbt
for ia:=1 to iat do anA[ia]:=1;
for ib:=1 to ibt do anB[ib]:=1;

vMin:=100000;
ib1:=1;
repeat
   // Compute one sum i.e. one product, matrix line times vector avX
   vSum:=0;
   for ia:=1 to iat do for ib:=1 to ibt do
   for ja:=1 to jat do for jb:=1 to jbt do begin
      if (anA[ia] = ja) and (anB[ib] = jb) then
           vSum:=vSum + (avPr[ia,ib,ja,jb] - 1) * avX[ia,ib,ja,jb]
      else vSum:=vSum + (avPr[ia,ib,ja,jb]    ) * avX[ia,ib,ja,jb]
      end;

   if (vMin > vSum) then vMin:=vSum;

   // Next possible realisation, i.e. next line
   ia1:=1;
   repeat
      fStopLoop:=TRUE;
      inc(anA[ia1]);
      if (anA[ia1] > jat) then begin
         anA[ia1]:=1;
         inc(ia1);
         fStopLoop:=FALSE;
         end;
      until fStopLoop or (ia1 > iat);

   if (ia1 > iat) then begin
      ib1:=1;
      repeat
         fStopLoop:=TRUE;
         inc(anB[ib1]);
         if (anB[ib1] > jbt) then begin
            anB[ib1]:=1;
            inc(ib1);
            fStopLoop:=FALSE;
            end;
         until fStopLoop or (ib1 > ibt);
      end;

   until (ib1 > ibt);

FuncF:=vMin;
end;  // TMyWindow.FuncF

procedure TMyWindow.Test3;
//========================
// 2 Settings, 3 possible results, with detector efficiency
var
  iat,ibt,jat,jbt : integer;
  ia,ib,ja,jb     : integer;
  ia1,ib1         : integer;
  fStopLoop       : boolean;
  anA,anB         : array[1..5] of integer; // Possible experiment results
  avThetaA,
  avThetaB    : array[1..5] of Float;
  avecSetA,
  avecSetB    : array[1..5] of RVectR3Float;
  vSign,
  vNoise,
  vLambda,
  vMean,
  vRealis     // One realisation
              : Float;
  vEffic      : Float;  // detector efficiency
  avEff       : array[1..3] of Float; // output probabilities

begin
iat:=2; // Number of settings
ibt:=2;
jat:=3;
jbt:=3;

// Defining the settings
avThetaA[1] := 0.0;
aVThetaA[2] := pi/2;
avThetaB[1] := pi/4;
aVThetaB[2] := 3*pi/4;

for ia:=1 to iat do
   avecSetA[ia]:=V3Assign(0.0, cos(avThetaA[ia]), sin(avThetaA[ia]));

for ib:=1 to ibt do
   avecSetB[ib]:=V3Assign(0.0, cos(avThetaB[ib]), sin(avThetaB[ib]));

// Noise in the entangled state
vNoise:=0.0;
vLambda:=1.0 - vNoise;

// Detector efficiency
vEffic:=0.83;

// result = 1 or 2 means that we got a click on exactly one detector
// result = 3      means that the detector didn't click
avEff[1]:=vEffic/2;
avEff[2]:=vEffic/2;
avEff[3]:=1.0 - vEffic;

// Defining the Probabilities
DBW(1, '--- Pr --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         if (ja < jat) and (jb < jbt) then begin
            if odd(ja+jb) then vSign:= 1
            else               vSign:=-1;
            avPr[ia,ib,ja,jb]:=avEff[ja]*avEff[jb]*(vLambda*(1.0 - vSign * avecSetA[ia]*avecSetB[ib]) + vNoise);
            end
         else avPr[ia,ib,ja,jb]:=avEff[ja]*avEff[jb];
         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avPr[ia,ib,ja,jb]));
         end;

// Marginal probabilities on Alice side
DBW(1, '--- P1 --------');
for ia:=1 to iat do
    for ja:=1 to jat do begin
      avP1[ia,ja]:=0;
      for jb:=1 to jbt do avP1[ia,ja]:=avP1[ia,ja] + avPr[ia,1,ja,jb];

      DBW(1, ITS5(ia) + ITS5(ja) + FTS(avP1[ia,ja]));
      end;

// Marginal probabilities on Bob side
DBW(1, '--- P2 --------');
for ib:=1 to ibt do
    for jb:=1 to jbt do begin
      avP2[ib,jb]:=0;
      for ja:=1 to jat do avP2[ib,jb]:=avP2[ib,jb] + avPr[1,ib,ja,jb];

      DBW(1, ITS5(ib) + ITS5(jb) + FTS(avP2[ib,jb]));
      end;

// Try to define the function of the random variables
DBW(1, '--- FF --------');
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         avFF[ia,ib,ja,jb]:=avPr[ia,ib,ja,jb] - avP1[ia,ja] * avP2[ib,jb];
//         avFF[ia,ib,ja,jb]:=avPr[ia,ib,ja,jb] / (avP1[ia,ja] * avP2[ib,jb]) -1.0;
// Pas bon         avFF[ia,ib,ja,jb]:=1 - (avP1[ia,ja] * avP2[ib,jb]) / avPr[ia,ib,ja,jb];
//         avFF[ia,ib,ja,jb]:=81*avPr[ia,ib,ja,jb] - 9;

         DBW(1, ITS5(ia) + ITS5(ib) + ITS5(ja) + ITS5(jb) +
                FTS(avFF[ia,ib,ja,jb]));
         end;

// Esperance of the combination of the random variables
vMean:=0;
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
         vMean:=vMean + avFF[ia,ib,ja,jb] * avPr[ia,ib,ja,jb];
         end;

// Comparaison between the Esperance and the realisations.
DBW(1, '-- Mean vs. Realis ---------');
for ia:=1 to iat do anA[ia]:=1;
for ib:=1 to ibt do anB[ib]:=1;

ib1:=1;
repeat
   // Compute one realisation
   vRealis:=0;
   for ia:=1 to iat do
      for ib:=1 to ibt do
         vRealis:=vRealis + avFF[ia,ib,anA[ia],anB[ib]];

   DBW(1, FTS(vMean) + FTS(vRealis));

   // Next possible realisation
   ia1:=1;
   repeat
      fStopLoop:=TRUE;
      inc(anA[ia1]);
      if (anA[ia1] > jat) then begin
         anA[ia1]:=1;
         inc(ia1);
         fStopLoop:=FALSE;
         end;
      until fStopLoop or (ia1 > iat);

   if (ia1 > iat) then begin
      ib1:=1;
      repeat
         fStopLoop:=TRUE;
         inc(anB[ib1]);
         if (anB[ib1] > jbt) then begin
            anB[ib1]:=1;
            inc(ib1);
            fStopLoop:=FALSE;
            end;
         until fStopLoop or (ib1 > ibt);
      end;

   until (ib1 > ibt);

DBW(1, 'min=' + FTS(FuncF(iat,ibt,jat,jbt,avFF)));
for ia:=1 to iat do
  for ib:=1 to ibt do
    for ja:=1 to jat do
      for jb:=1 to jbt do begin
        avFF[ia,ib,ja,jb]:=avFF[ia,ib,ja,jb] + 0.08;
        DBW(1, 'min=' + FTS(FuncF(iat,ibt,jat,jbt,avFF)));
        avFF[ia,ib,ja,jb]:=avFF[ia,ib,ja,jb] - 0.08;
        end;
end;  // TMyWindow.Test3

procedure TMyWindow.Test4;
//========================
begin
end;  // TMyWindow.Test4

function TMyWindow.WMCommand(wPar:WPARAM; lPar:LPARAM):LRESULT;
//=============================================================
// Handles the menu messages
Var wNrMenu     : word;
begin
WMCommand := 0; // Default return value when processing a message

if (lPar <> 0) then begin
   // call the parent WMCommand method.
   WMCommand:=inherited WMCommand(wPar, lPar);
   exit;
   end;

// for the menus,  lPar = 0
//=========================
wNrMenu   := LoWord(wPar);
Case wNrMenu Of
   101 : Test1;
   102 : Test2;
   103 : Test3;
   104 : Test4;

   else WMCommand:=inherited WMCommand(wPar, lPar);
   end;  // Case NrMenu Of
end;  // TMyWindow.WMCommand

function TMyWindow.WindowProc(hWindow:HWND; wMsg:UINT; wPar:WPARAM; lPar:LPARAM):LRESULT;
//======================================================================================
begin
// Default treatment of a message. Does nearly nothing
WindowProc := inherited WindowProc(hWindow, wMsg, wPar, lPar);
end;  // TMyWindow.WindowProc

Begin // MAIN ******
// Name under which the main window is registered. Spaces are not allowed
oApp.Init('tzNonLocal01');

// Creation of the main window
new(poMyMainWin, Init('tzNonLocal01, to characterise non locality'));

oApp.Run;
oApp.Done;  // The main window object is disposed here.

glf:=glf; // To avoid a Hint
End.

